package org.rcisoft.business.crawler.service.impl;

import cn.hutool.core.util.StrUtil;
import io.github.furstenheim.CopyDown;
import io.github.furstenheim.Options;
import io.github.furstenheim.OptionsBuilder;
import lombok.Cleanup;
import lombok.SneakyThrows;
import org.apache.commons.io.FilenameUtils;
import org.commonmark.node.Node;
import org.commonmark.parser.Parser;
import org.commonmark.renderer.html.HtmlRenderer;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Entities;
import org.jsoup.select.Elements;
import org.rcisoft.business.cmsnews.entity.CmsNews;
import org.rcisoft.business.cmsnews.service.CmsNewsService;
import org.rcisoft.business.crawler.model.Article;
import org.rcisoft.business.crawler.service.WebsiteCrawlerService;
import org.rcisoft.business.crawler.utils.SSLUtilities;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.entity.CyFileEntity;
import org.rcisoft.core.service.CyCommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 定时爬虫业务层
 * 爬取网站：https://www.iachina.cn/col/col23/index.html
 *
 * @author gxk
 */
@Service
public class WebsiteCrawlerServiceImpl implements WebsiteCrawlerService {

    public static final String MP_WEIXIN_QQ_COM = "https://mp.weixin.qq.com/";
    public static final String PROPERTY_PICC_COM = "https://property.picc.com/";
    public static final String FINANCE_CNR_CN = "https://finance.cnr.cn/";
    public static final String HSBCINSURANCE_COM_CN = "https://www.hsbcinsurance.com.cn/";
    public static final String FINANCE_PEOPLE_COM_CN = "http://finance.people.com.cn/";
    public static final String FINANCE_CHINA_COM_CN = "http://finance.china.com.cn/";
    public static final String NEWS_CN = "http://www.news.cn/";
    public static final String CS_COM_CN = "https://www.cs.com.cn/";
    public static final String CBIMC_CN = "http://www.cbimc.cn/";
    public static final String E_CHINALIFE_COM = "https://www.e-chinalife.com/";
    public static final String PEOPLEAPP_COM = "https://www.peopleapp.com/";

    /**
     * 指定URL
     */
    public static String url = "https://www.iachina.cn/col/col23/index.html";

    /**
     * 指定日期
     */
    public static String date = "2024-01-01";

    @Autowired
    private CmsNewsService cmsNewsService;

    @Autowired
    private CyCommonService cyCommonService;

    @Autowired
    private Global global;

    /**
     * 根据String类型的url转为MultipartFile
     *
     * @param fileUrl
     * @return
     * @throws IOException
     */
    @SneakyThrows
    public static MultipartFile convertUrlToMultipartFile(String fileUrl, String newFileName) {
        // 禁用HTTPS证书验证
        SSLUtilities.disableSSLVerification();
        // 从URL下载文件
        URL url = new URL(fileUrl);
        @Cleanup InputStream inputStream = url.openStream();
        byte[] fileBytes = inputStream.readAllBytes();

        // 使用MockMultipartFile创建MultipartFile对象
        return new MockMultipartFile(newFileName, newFileName, "image/jpeg", fileBytes);
    }

    public static String generateRandomFileName(String fileUrl) {
        String extension = getExtensionFromUrl(fileUrl);
        if (extension.isEmpty()) {
            extension = "jpg";
        }
        return UUID.randomUUID().toString() + "." + extension;
    }

    public static String getExtensionFromUrl(String fileUrl) {
        // 从URL获取文件名
        String fileName = FilenameUtils.getName(fileUrl.split("\\?")[0]);
        // 获取文件扩展名
        return FilenameUtils.getExtension(fileName);
    }

    /**
     * html -> markdown
     *
     * @param html
     * @return
     */
    public static String convertHtmlToMarkdown(String html) {
        OptionsBuilder optionsBuilder = OptionsBuilder.anOptions();
        Options options = optionsBuilder.withBr("-")
                .build();
        CopyDown converter = new CopyDown(options);
        return converter.convert(html);
    }

    /**
     * markdown -> html
     *
     * @param markdownContent
     * @return
     */
    public static String convertMarkdownToHtml(String markdownContent) {
        // 创建 Markdown 解析器，解析 Markdown
        Parser parser = Parser.builder().build();
        Node document = parser.parse(markdownContent);

        // 创建 HTML 渲染器
        HtmlRenderer renderer = HtmlRenderer.builder().build();

        // 渲染 HTML
        org.jsoup.nodes.Document doc = Jsoup.parse(renderer.render(document).replaceAll("\\\\n", "<br>"));

        // jsoup标准化标签，生成闭合标签
        doc.outputSettings().syntax(org.jsoup.nodes.Document.OutputSettings.Syntax.xml);
        doc.outputSettings().escapeMode(Entities.EscapeMode.xhtml);
        return doc.body().html();
    }

    /**
     * mp.weixin.qq.com
     *
     * @return
     */
    public static Map<String, String> getWeiXin(Article article, Document document) throws IOException {
        Map<String, String> map = new HashMap<>();
        String title = document.select("h1.rich_media_title").html();
        Element elementById = document.getElementById("img-content");
        if (elementById != null) {
            String content = Objects.requireNonNull(document.getElementById("img-content")).html();
            map.put("title", title);
            map.put("content", content.replaceAll("data-src", "src"));
        } else {
            //标题截取
            map.put("title", article.getTitle());
//            System.out.println(Jsoup.connect(article.getTitle()).get());
            map.put("content", article.getTitle());
        }
        return map;
    }


    /**
     * www.cs.com.cn
     *
     * @return
     */
    public static Map<String, String> getCsComCn(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("article.cont_article header h1").html();
        String content = document.select("article.cont_article").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * www.cbimc.cn
     *
     * @return
     */
    public static Map<String, String> getCbimcCn(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("div.detail-h h1").html();
        String content = document.select("div.left-l").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * www.news.cn
     *
     * @return
     */
    public static Map<String, String> getNewsCn(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("span.title").html();
        String titleContent = document.select("div.header-cont").html();
        String content = Objects.requireNonNull(document.getElementById("detailContent")).html();
        map.put("title", title);
        map.put("content", titleContent + content);
        return map;
    }

    /**
     * e-chinalife.com
     *
     * @return
     */
    public static Map<String, String> getChinalife(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("div.darticle h2.aTitle").html();
        String content = document.select("div.darticle").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * property.picc.com
     *
     * @return
     */
    public static Map<String, String> getPiccCom(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("h1.news-infoBoxTitle").html();
        String content = document.select("div.news-infoBox-content").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * finance.cnr.cn
     *
     * @return
     */
    public static Map<String, String> getFinanceCnr(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("div.article-header h1").html();
        String titleContent = document.select("div.article-header").html();
        String content = document.select("div.article-content").html();
        map.put("title", title);
        map.put("content", titleContent + content);
        return map;
    }

    /**
     * www.hsbcinsurance.com.cn
     *
     * @return
     */
    public static Map<String, String> getHsbcinsurance(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("span.A-TYP16B-RW-ALL span").html();
        String content = document.select("div.cc-column").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * finance.people.com.cn
     *
     * @return
     */
    public static Map<String, String> getFinancePeople(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("div.col.col-1.fl h1").html();
        String content = document.select("div.col.col-1.fl").html();
        map.put("title", title);
        map.put("content", content);
        return map;
    }

    /**
     * finance.china.com.cn
     *
     * @return
     */
    public static Map<String, String> getFinanceChina(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("h1.toph1").html();
        String titleContent = document.select("div.wrap.c.top").html();
        String content = document.select("div.fl.navl").html();
        map.put("title", title);
        map.put("content", titleContent + content);
        return map;
    }

    private static Map<String, String> getPeopleAppCom(Document document) {
        Map<String, String> map = new HashMap<>();
        String title = document.select("div.title").html();
        String content = document.select("body").html();
        map.put("title", title);
        map.put("content", content);
        return map;

    }

    /**
     * 开始爬取指定网站数据
     */
    @SneakyThrows
    @Override
    public Integer startCrawler() throws Exception {
        // 通过网址获取Document信息
        Document document = Jsoup.connect(url).get();
        String html = document.html();

        // 正则拿取隐藏的信息
        String regexDiv = "<record><!\\[CDATA\\[(.*?)]]></record>";
        Pattern pattern = Pattern.compile(regexDiv, Pattern.DOTALL);
        Matcher matcher = pattern.matcher(html);
        List<String> list = new ArrayList<>();
        //这个地方控制当前爬取的文章数量，字典表中必须得有没有不爬删了也不爬 目前web端只有一个已经写死所以
        //查询字典  还要在别的地方用所以 字段传入不写死 第一个传入网站名称，第二个传入 控制的字典词  死值hōtori_num
        Integer dictionaryValue = cmsNewsService.selectDictionaryControl("中国保险行业协会 行业要闻", "hōtori_num");
        // 把本年的文章拿取
        while (matcher.find()) {
            String group = matcher.group(1);
            String nowDate = Jsoup.parse(group).select("div.date").text();
            int i = date.compareTo(nowDate);
            if (i < 0) {
                list.add(matcher.group(1));
            }
        }
        //如果要截取得长度大于获取长度
        if (list.size() > dictionaryValue) {
            //list 从头按顺序 截取 爬取数量条
            list = list.subList(0, dictionaryValue);
        }
        //计数器
        int count = 0;
        for (String item : list) {
            Map<String, String> map = new HashMap<>(2);
            // 获取文章链接
            String articleUrl = Jsoup.parse(item).getElementsByTag("a").attr("href");

            // 设置随机延迟时间
            Random random = new Random();
            int minDelay = 5000;
            int maxDelay = 10000;
            int delay = minDelay + random.nextInt(maxDelay - minDelay + 1);
            Thread.sleep(delay);
            Document doc = Jsoup.connect(articleUrl).timeout(10000).get();

            // 根据网址不同，解析方式不同
            if (articleUrl.contains(MP_WEIXIN_QQ_COM)) {
                map = getWeiXin(null, doc);
            } else if (articleUrl.contains(CS_COM_CN)) {
                map = getCsComCn(doc);
            } else if (articleUrl.contains(CBIMC_CN)) {
                map = getCbimcCn(doc);
            } else if (articleUrl.contains(NEWS_CN)) {
                map = getNewsCn(doc);
            } else if (articleUrl.contains(E_CHINALIFE_COM)) {
                map = getChinalife(doc);
            } else if (articleUrl.contains(PROPERTY_PICC_COM)) {
                map = getPiccCom(doc);
            } else if (articleUrl.contains(FINANCE_CNR_CN)) {
                map = getFinanceCnr(doc);
            } else if (articleUrl.contains(HSBCINSURANCE_COM_CN)) {
                map = getHsbcinsurance(doc);
            } else if (articleUrl.contains(FINANCE_PEOPLE_COM_CN)) {
                map = getFinancePeople(doc);
            } else if (articleUrl.contains(FINANCE_CHINA_COM_CN)) {
                map = getFinanceChina(doc);
            } else if (articleUrl.contains(PEOPLEAPP_COM)) {
                map = getPeopleAppCom(doc);
            }
            //通过 title 判断当前文章是否跟数据库有重复
            String title = cmsNewsService.getNewsByTitleByTitle(map.get("title"));
            if (title == null && map.containsKey("title") && map.containsKey("content")) {
                // 图片转换
                Document parse = Jsoup.parse(map.get("content"));
                replaceImgSrc(parse);
                // html 转 markdown, 再转回来
                String markdown = convertHtmlToMarkdown(parse.html());
                String content = convertMarkdownToHtml(markdown);
                CmsNews cmsNew = CmsNews.builder()
                        .releaseDate(new Date())
                        .title(map.get("title"))
                        .details(content)
                        .isRecommended(0)
                        .isTop(0)
                        .newsType("0")
                        .build();
                cmsNew.setFlag("1");
                // 插入文章
                cmsNewsService.persist(cmsNew);
                count++;
            }

        }
        return count;

    }

    /**
     * 图片转换，防止盗链
     *
     * @param doc
     */
    @SneakyThrows
    public void replaceImgSrc(Document doc) {
        // 选择所有img标签
        Elements imgElements = doc.getElementsByTag("img");
        // 遍历所有img标签并替换src属性
        for (Element imgElement : imgElements) {
            String url = imgElement.attr("src");
            if (StrUtil.isNotEmpty(url) && url.contains("https://")) {
                String randomFileName = generateRandomFileName(url);
                MultipartFile multipartFile = convertUrlToMultipartFile(url, randomFileName);
                CyFileEntity cyFileEntity = cyCommonService.filePath(multipartFile, "crawler");
                // 替换src属性为本地路径
                imgElement.attr("src", global.getBase_Discovery() + cyFileEntity.getPath());
            }
        }
    }

}
